<?php

namespace XF2SEO\SEOPackage\Admin\Controller;

use XF\Mvc\FormAction;
use XF\Mvc\ParameterBag;
use XF\ControllerPlugin\Delete;
use XF\Mvc\Reply\AbstractReply;
use XF\Admin\Controller\AbstractController;

class Redirect extends AbstractController
{
    public function actionIndex()
    {
        $finder = $this->finder('XF2SEO\SEOPackage:Redirect');

        $page = $this->filterPage();
        $perPage = 30;

        $total = $finder->total();
        $this->assertValidPage($page, $perPage, $total, $this->getLinkPrefix());

        $redirects = $finder->limitByPage($page, $perPage)->fetch();

        return $this->view(
            '',
            'seo_package_redirect_list',
            [
                'redirects' => $redirects,
                'linkPrefix' => $this->getLinkPrefix(),
                'page' => $page,
                'perPage' => $perPage,
                'total' => $total,
            ]
        );
    }

    public function actionAdd()
    {
        return $this->getRedirectForm($this->getNewRedirect());
    }

    public function actionEdit(ParameterBag $params)
    {
        $keyword = $this->assertRedirectExists($params['redirect_id']);

        return $this->getRedirectForm($keyword);
    }

    public function actionSave(ParameterBag $params)
    {
        if ($params['redirect_id'] > 0) {
            $redirect = $this->assertRedirectExists($params['redirect_id']);
        } else {
            $redirect = $this->getNewRedirect();
        }

        $this->saveRedirectProcess($redirect)->run();

        return $this->redirect(
            $this->buildLink($this->getLinkPrefix())
            . $this->buildLinkHash($redirect->redirect_id)
        );
    }

    public function actionDelete(ParameterBag $params)
    {
        $redirect = $this->assertRedirectExists($params['redirect_id']);

        /** @var Delete $deletePlugin */
        $deletePlugin = $this->plugin('XF:Delete');

        return $deletePlugin->actionDelete(
            $redirect,
            $this->buildLink($this->getLinkPrefix() . '/delete', $redirect),
            $this->buildLink($this->getLinkPrefix() . '/edit', $redirect),
            $this->buildLink($this->getLinkPrefix()),
            $redirect->match_url
        );
    }

    protected function getLinkPrefix(): string
    {
        return 'tsp-redirects';
    }

    protected function saveRedirectProcess(\XF2SEO\SEOPackage\Entity\Redirect $redirect): FormAction
    {
        $formAction = $this->formAction();

        $input = $this->filter([
            'match_url' => 'str',
            'target_url' => 'str',
            'active' => 'bool',
            'redirect_type' => 'str',
        ]);

        $formAction->basicEntitySave($redirect, $input);

        return $formAction;
    }

    protected function getNewRedirect(): \XF2SEO\SEOPackage\Entity\Redirect
    {
        /** @var \XF2SEO\SEOPackage\Entity\Redirect $redirect */
        $redirect = $this->em()->create('XF2SEO\SEOPackage:Redirect');

        return $redirect;
    }

    protected function getRedirectForm(\XF2SEO\SEOPackage\Entity\Redirect $redirect): AbstractReply
    {
        return $this->view(
            '',
            'seo_package_redirect_edit',
            [
                'redirect' => $redirect,
                'linkPrefix' => $this->getLinkPrefix(),
            ]
        );
    }

    /**
     * @param mixed $id
     * @return \XF2SEO\SEOPackage\Entity\Redirect
     * @throws \XF\Mvc\Reply\Exception
     */
    protected function assertRedirectExists($id): \XF2SEO\SEOPackage\Entity\Redirect
    {
        /** @var \XF2SEO\SEOPackage\Entity\Redirect $redirect */
        $redirect = $this->assertRecordExists(
            'XF2SEO\SEOPackage:Redirect',
            $id
        );

        return $redirect;
    }
}
